////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//                   LED CLOCK "SHOOT IN THE FOOT" PROJECT                    //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

// I2C Unit ////////////////////////////////////////////////////////////////////

sbit Soft_I2C_Scl           at RC5_bit;
sbit Soft_I2C_Sda           at RC4_bit;
sbit Soft_I2C_Scl_Direction at TRISC5_bit;
sbit Soft_I2C_Sda_Direction at TRISC4_bit;

unsigned char Second, Minute, Hour;

unsigned char RTC_Read(unsigned char address)
{
  unsigned char value = 0;
  Soft_I2C_Start();
  Soft_I2C_Write(0xD0);
  Soft_I2C_Write(address);
  Soft_I2C_Start();
  Soft_I2C_Write(0xD1);
  value = Soft_I2C_Read(0);
  Soft_I2C_Stop();
  return value;
}

void RTC_Write(unsigned char address, unsigned char value)
{
  Soft_I2C_Start();
  Soft_I2C_Write(0xD0);
  Soft_I2C_Write(address);
  Soft_I2C_Write(value);
  Soft_I2C_Stop();
}

void RTC_init()
{
  //RTC_Write(0x00, 0x59);    // Second
  //RTC_Write(0x01, 0x37);    // Minute
  //RTC_Write(0x02, 0x12);    // Hour
  RTC_Write(0x0E, 0x40);    // CTRL
  RTC_Write(0x0F, 0x00);    // CTRL STA
  RTC_Write(0x03, 0x01);    // Day (01-07)
  RTC_Write(0x04, 0x01);    // Day (01-31)
  RTC_Write(0x05, 0x01);    // Month (01-12)
  RTC_Write(0x06, 0x01);    // Year (00-99)
}

// Serial Display Primary //////////////////////////////////////////////////////

#define SHIFT_DAT_1 PORTB.F5
#define SHIFT_CLC_1 PORTB.F7
#define SHIFT_LAT_1 PORTB.F6

void Shift_latch_Primary()
{
  SHIFT_LAT_1 = 1;                      // Lat
  asm nop;
  SHIFT_LAT_1 = 0;                      // Lat
}

void Shift_out_Primary(char one_digit)
{
  char x;
  for (x=0x00; x<=0x07; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_1 = 1;                  // Dat
    else SHIFT_DAT_1 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_1 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_1 = 0;                    // Clk
  }
}

void Shift_out_Primary_1(char one_digit)
{
  char x;
  for (x=0x00; x<=0x00; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_1 = 1;                  // Dat
    else SHIFT_DAT_1 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_1 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_1 = 0;                    // Clk
  }
}

void Shift_out_Primary_2(char one_digit)
{
  char x;
  for (x=0x00; x<=0x01; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_1 = 1;                  // Dat
    else SHIFT_DAT_1 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_1 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_1 = 0;                    // Clk
  }
}

void Shift_out_Primary_3(char one_digit)
{
  char x;
  for (x=0x00; x<=0x02; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_1 = 1;                  // Dat
    else SHIFT_DAT_1 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_1 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_1 = 0;                    // Clk
  }
}

void Shift_out_Primary_4(char one_digit)
{
  char x;
  for (x=0x00; x<=0x03; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_1 = 1;                  // Dat
    else SHIFT_DAT_1 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_1 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_1 = 0;                    // Clk
  }
}

unsigned char Digit4HH(int a)           // Digit - to - port
{
  switch(a)
  {
                      //FBAG
    case 0x00: return 0x10;
    case 0x01: return 0xB0;
    case 0x02: return 0x80;
    case 0x03: return 0x80;
    case 0x04: return 0x20;
    case 0x05: return 0x40;
    case 0x06: return 0x40;
    case 0x07: return 0x90;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit3HL(int a)           // Digit - to - port
{
  switch(a)
  {
                      //AFB
    case 0x00: return 0x00;
    case 0x01: return 0xC0;
    case 0x02: return 0x40;
    case 0x03: return 0x40;
    case 0x04: return 0x80;
    case 0x05: return 0x20;
    case 0x06: return 0x20;
    case 0x07: return 0x40;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit1ML(int a)           // Digit - to - port
{
  switch(a)
  {
                      //A
    case 0x00: return 0x00;
    case 0x01: return 0x80;
    case 0x02: return 0x00;
    case 0x03: return 0x00;
    case 0x04: return 0x80;
    case 0x05: return 0x00;
    case 0x06: return 0x00;
    case 0x07: return 0x00;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit4MH(int a)           // Digit - to - port
{
  switch(a)
  {
                      //BAGF
    case 0x00: return 0x20;
    case 0x01: return 0x70;
    case 0x02: return 0x10;
    case 0x03: return 0x10;
    case 0x04: return 0x40;
    case 0x05: return 0x80;
    case 0x06: return 0x80;
    case 0x07: return 0x30;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit2ML(int a)           // Digit - to - port
{
  switch(a)
  {
                      //FB
    case 0x00: return 0x00;
    case 0x01: return 0x80;
    case 0x02: return 0x80;
    case 0x03: return 0x80;
    case 0x04: return 0x00;
    case 0x05: return 0x40;
    case 0x06: return 0x40;
    case 0x07: return 0x80;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit4ML(int a)           // Digit - to - port
{
  switch(a)
  {
                      //CEDG
    case 0x00: return 0x10;
    case 0x01: return 0x70;
    case 0x02: return 0x80;
    case 0x03: return 0x40;
    case 0x04: return 0x60;
    case 0x05: return 0x40;
    case 0x06: return 0x00;
    case 0x07: return 0x70;
    case 0x08: return 0x00;
    case 0x09: return 0x40;
  }
}

unsigned char Digit3MH(int a)           // Digit - to - port
{
  switch(a)
  {
                      //DCE
    case 0x00: return 0x00;
    case 0x01: return 0xA0;
    case 0x02: return 0x40;
    case 0x03: return 0x20;
    case 0x04: return 0xA0;
    case 0x05: return 0x20;
    case 0x06: return 0x00;
    case 0x07: return 0xA0;
    case 0x08: return 0x00;
    case 0x09: return 0x20;
  }
}

unsigned char Digit1HH(int a)           // Digit - to - port
{
  switch(a)
  {
                      //D
    case 0x00: return 0x00;
    case 0x01: return 0x80;
    case 0x02: return 0x00;
    case 0x03: return 0x00;
    case 0x04: return 0x80;
    case 0x05: return 0x00;
    case 0x06: return 0x00;
    case 0x07: return 0x80;
    case 0x08: return 0x00;
    case 0x09: return 0x00;
  }
}

unsigned char Digit4HL(int a)           // Digit - to - port
{
  switch(a)
  {
                      //EDGC
    case 0x00: return 0x20;
    case 0x01: return 0xE0;
    case 0x02: return 0x10;
    case 0x03: return 0x80;
    case 0x04: return 0xC0;
    case 0x05: return 0x80;
    case 0x06: return 0x00;
    case 0x07: return 0xE0;
    case 0x08: return 0x00;
    case 0x09: return 0x80;
  }
}

unsigned char Digit2HH(int a)           // Digit - to - port
{
  switch(a)
  {
                      //CE
    case 0x00: return 0x00;
    case 0x01: return 0x40;
    case 0x02: return 0x80;
    case 0x03: return 0x40;
    case 0x04: return 0x40;
    case 0x05: return 0x40;
    case 0x06: return 0x00;
    case 0x07: return 0x40;
    case 0x08: return 0x00;
    case 0x09: return 0x40;
  }
}

void Show_Time()
{
  Shift_out_Primary_2(Digit2HH((Hour/0x10)%0x10));    // 2HH
  Shift_out_Primary_4(Digit4HL(Hour%0x10));           // 4HL
  Shift_out_Primary_1(Digit1HH((Hour/0x10)%0x10));    // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(Digit3MH((Minute/0x10)%0x10));  // 3MH
  Shift_out_Primary_4(Digit4ML(Minute%0x10));         // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(Digit2ML(Minute%0x10));         // 2ML
  Shift_out_Primary_4(Digit4MH((Minute/0x10)%0x10));  // 4MH
  Shift_out_Primary_1(Digit1ML(Minute%0x10));         // 1ML
  Shift_out_Primary_1(0xFF);                          // 1LL
  Shift_out_Primary_3(Digit3HL(Hour%0x10));           // 3HL
  Shift_out_Primary_4(Digit4HH((Hour/0x10)%0x10));    // 4HH
  Shift_out_Primary_1(0xFF);                          // 1LH
  Shift_latch_Primary();
}

// Serial Display Secondary ////////////////////////////////////////////////////

#define SHIFT_DAT_2 PORTC.F3
#define SHIFT_CLC_2 PORTC.F1
#define SHIFT_LAT_2 PORTC.F2

void Shift_latch_Secondary()
{
  SHIFT_LAT_2 = 1;                      // Lat
  asm nop;
  SHIFT_LAT_2 = 0;                      // Lat
}

unsigned char Digit_Ring(int a)         // Digit - to - port
{
  switch(a)
  {
    case 0x00: return 0x00;  //-
    case 0x01: return 0x01;
    case 0x02: return 0x02;
    case 0x03: return 0x03;
    case 0x04: return 0x04;
    case 0x05: return 0x05;
    case 0x06: return 0x06;
    case 0x07: return 0x07;
    case 0x08: return 0x08;
    case 0x09: return 0x09;
    case 0x10: return 0x0A;
    case 0x11: return 0x0B;
    case 0x12: return 0x0C;
    case 0x13: return 0x0D;
    case 0x14: return 0x0E;  //-
    case 0x15: return 0x10;
    case 0x16: return 0x11;
    case 0x17: return 0x12;
    case 0x18: return 0x13;
    case 0x19: return 0x14;
    case 0x20: return 0x15;
    case 0x21: return 0x16;
    case 0x22: return 0x17;
    case 0x23: return 0x18;
    case 0x24: return 0x19;
    case 0x25: return 0x1A;
    case 0x26: return 0x1B;
    case 0x27: return 0x1C;
    case 0x28: return 0x1D;
    case 0x29: return 0x1E;
    case 0x30: return 0x1F;  //-
    case 0x31: return 0x01;
    case 0x32: return 0x02;
    case 0x33: return 0x03;
    case 0x34: return 0x04;
    case 0x35: return 0x05;
    case 0x36: return 0x06;  //-
    case 0x37: return 0x08;
    case 0x38: return 0x09;
    case 0x39: return 0x0A;
    case 0x40: return 0x0B;
    case 0x41: return 0x0C;
    case 0x42: return 0x0D;
    case 0x43: return 0x0E;
    case 0x44: return 0x0F;
    case 0x45: return 0x10;
    case 0x46: return 0x11;
    case 0x47: return 0x12;
    case 0x48: return 0x13;
    case 0x49: return 0x14;
    case 0x50: return 0x15;
    case 0x51: return 0x16;  //-
    case 0x52: return 0x18;
    case 0x53: return 0x19;
    case 0x54: return 0x1A;
    case 0x55: return 0x1B;
    case 0x56: return 0x1C;
    case 0x57: return 0x1D;
    case 0x58: return 0x1E;
    case 0x59: return 0x1F;
  }
}

void Shift_out_Secondary_1b(char one_digit)
{
  char x;
  for (x=0x00; x<=0x00; ++x)            // DTL int
  {
    if (one_digit & 0x80)               // DTL mask
      SHIFT_DAT_2 = 1;                  // Dat
    else SHIFT_DAT_2 = 0;               // Dat
    one_digit <<= 1;
    SHIFT_CLC_2 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_2 = 0;                    // Clk
  }
}

void Shift_out_Secondary_L(char one_digit)
{
  char x;
  for (x=0x00; x<=0x1E; ++x)            // DTL int
  {
    one_digit --;
    if (one_digit <= 0x40)              // DTL mask
      SHIFT_DAT_2 = 1;                  // Dat
    else
      SHIFT_DAT_2 = 0;                  // Dat
    SHIFT_CLC_2 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_2 = 0;                    // Clk
  }
}

void Shift_out_Secondary_H(char one_digit)
{
  char x;
  for (x=0x00; x<=0x1E; ++x)            // DTL int
  {
    one_digit --;
    if (one_digit <= 0x40)              // DTL mask
      SHIFT_DAT_2 = 0;                  // Dat
    else
      SHIFT_DAT_2 = 1;                  // Dat
    SHIFT_CLC_2 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_2 = 0;                    // Clk
  }
}

void Shift_out_Secondary_R(char one_digit)
{
  char x;
  for (x=0x00; x<=0x1E; ++x)            // DTL int
  {
    one_digit --;
    if (one_digit != 0)                 // DTL mask
      SHIFT_DAT_2 = 1;                  // Dat
    else
      SHIFT_DAT_2 = 0;                  // Dat
    SHIFT_CLC_2 = 1;                    // Clk
    asm nop;
    SHIFT_CLC_2 = 0;                    // Clk
  }
}

void ShowSecond_L()
{
  if (Second == 0x00)
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
  }
  if (Second == 0x30)
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x00);
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x1F);
  }
  if ((Second < 0x30) && (Second != 0x00))
  {
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x00);
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(Digit_Ring(Second));
  }
  if (Second > 0x30)
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(Digit_Ring(Second));
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x1F);
  }
  Shift_latch_Secondary();
}

void ShowSecond_H()
{
  if (Second == 0x00)
  {
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x00);
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x00);
  }
  if (Second == 0x30)
  {
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x00);
  }
  if ((Second < 0x30) && (Second != 0))
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_H(Digit_Ring(Second));
  }
  if (Second > 0x30)
  {
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_H(Digit_Ring(Second));
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x00);
  }
  Shift_latch_Secondary();
}

void ShowSecond_R()
{
  if (Second == 0x00)
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x1F);
  }
  if (Second == 0x30)
  {
    Shift_out_Secondary_1b(0x00);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
  }
  if ((Second < 0x30) && (Second != 0))
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_R(Digit_Ring(Second));
  }
  if (Second > 0x30)
  {
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_R(Digit_Ring(Second));
    Shift_out_Secondary_1b(0xFF);
    Shift_out_Secondary_L(0x1F);
  }
  Shift_latch_Secondary();
}

void Show_Second()
{
  if ((Minute%0x10 == 0x05) || (Minute%0x10 == 0x00))
  {
    if (Second == 0x00)
      ShowSecond_L();
    else
      ShowSecond_R();
  }
  if ((Minute%0x10 == 0x01) || (Minute%0x10 == 0x06))
  {
    if (Second == 0x00)
      ShowSecond_R();
    else
      ShowSecond_H();
  }
  if ((Minute%0x10 == 0x03) || (Minute%0x10 == 0x08))
  {
    if (Second == 0x00)
      ShowSecond_L();
    else
      ShowSecond_H();
  }
  if ((Minute%0x10 == 0x02) || (Minute%0x10 == 0x04) || (Minute%0x10 == 0x07) || (Minute%0x10 == 0x09))
  {
    if (Second == 0x00)
      ShowSecond_H();
    else
      ShowSecond_L();
  }
}

// DHT Unit ////////////////////////////////////////////////////////////////////

#define DHT_IO PORTC.F6
#define DHT_IO_DIRECTION TRISC.F6

unsigned char T_byte1, T_byte2, RH_byte1, RH_byte2;
unsigned int Temp, RH, Sum;
bit Check, Minus;

void DHT_StartSignal()
{
  DHT_IO_DIRECTION = 0;
  DHT_IO = 0;
  delay_ms(18);
  DHT_IO = 1;
  delay_us(30);
  DHT_IO_DIRECTION = 1;
}

void DHT_CheckResponse()
{
  Check = 0;
  delay_us(40);
  if (DHT_IO == 0)
  {
    delay_us(80);
    if (DHT_IO == 1)
    {
      Check = 1;
      delay_us(40);
    }
  }
}

char DHT_Read()
{
  char i, j;
  for(j = 0; j < 8; j++)
  {
    while(!DHT_IO);
    delay_us(30);
    if(DHT_IO == 0)
      i&= ~(1<<(7 - j));
    else
    {
      i|= (1 << (7 - j));
      while(DHT_IO);
    }
  }
  return i;
}

void DHT_Read_Data()
{
  DHT_StartSignal();
  DHT_CheckResponse();
  if(Check == 1)
  {
    RH_byte1 = DHT_Read();
    RH_byte2 = DHT_Read();
    T_byte1 = DHT_Read();
    T_byte2 = DHT_Read();
    Sum = DHT_Read();
    if(Sum == ((RH_byte1+RH_byte2+T_byte1+T_byte2) & 0XFF))
    {
      Minus = 0;
      Temp = T_byte1;
      Temp = (Temp << 8) | T_byte2;
      RH = RH_byte1;
      RH = (RH << 8) | RH_byte2;
      if (Temp > 0x8000)
      {
        Temp = Temp & 0x7FFF;
        Minus = 1;
      }
    }
    else
    {
      Check = 0;
    }
  }
  else
  {
    Check = 0;
  }
}

void DHT_Show_Error()                                 // --8--
{
  Shift_out_Primary_2(0xC0);                          // 2HH
  Shift_out_Primary_4(0xD0);                          // 4HL
  Shift_out_Primary_1(0x80);                          // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(0xE0);                          // 3MH
  Shift_out_Primary_4(0xE0);                          // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(0xC0);                          // 2ML
  Shift_out_Primary_4(0xD0);                          // 4MH
  Shift_out_Primary_1(0x80);                          // 1ML
  Shift_out_Primary_1(0xFF);                          // 1LL
  Shift_out_Primary_3(0xE0);                          // 3HL
  Shift_out_Primary_4(0xE0);                          // 4HH
  Shift_out_Primary_1(0xFF);                          // 1LH
  Shift_latch_Primary();
}

void DHT_Show_RH()                                    // XX **
{
  if (RH > 0x03E7)                                    // 99.9
  {
    RH = 0x03E7;
  }
  Shift_out_Primary_2(Digit2HH((RH/0x64)%0x0A));      // 2HH
  Shift_out_Primary_4(Digit4HL((RH/0x0A)%0x0A));      // 4HL
  Shift_out_Primary_1(Digit1HH((RH/0x64)%0x0A));      // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(0xE0);                          // 3MH
  Shift_out_Primary_4(0x00);                          // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(0xC0);                          // 2ML
  Shift_out_Primary_4(0x00);                          // 4MH
  Shift_out_Primary_1(0x80);                          // 1ML
  Shift_out_Primary_1(0x00);                          // 1LL
  Shift_out_Primary_3(Digit3HL((RH/0x0A)%0x0A));      // 3HL
  Shift_out_Primary_4(Digit4HH((RH/0x64)%0x0A));      // 4HH
  Shift_out_Primary_1(0x00);                          // 1LH
  Shift_latch_Primary();
}

void DHT_Show_Temp_H()                                // XX *C
{
  Shift_out_Primary_2(Digit2HH((Temp/0x64)%0x0A));    // 2HH
  Shift_out_Primary_4(Digit4HL((Temp/0x0A)%0x0A));    // 4HL
  Shift_out_Primary_1(Digit1HH((Temp/0x64)%0x0A));    // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(0xE0);                          // 3MH
  Shift_out_Primary_4(0x90);                          // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(0x40);                          // 2ML
  Shift_out_Primary_4(0x00);                          // 4MH
  Shift_out_Primary_1(0x00);                          // 1ML
  Shift_out_Primary_1(0x00);                          // 1LL
  Shift_out_Primary_3(Digit3HL((Temp/0x0A)%0x0A));    // 3HL
  Shift_out_Primary_4(Digit4HH((Temp/0x64)%0x0A));    // 4HH
  Shift_out_Primary_1(0x00);                          // 1LH
  Shift_latch_Primary();
}

void DHT_Show_Temp_L()                                // -X *C
{
  Shift_out_Primary_2(0xC0);                          // 2HH
  Shift_out_Primary_4(Digit4HL((Temp/0x0A)%0x0A));    // 4HL
  Shift_out_Primary_1(0x80);                          // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(0xE0);                          // 3MH
  Shift_out_Primary_4(0x90);                          // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(0x40);                          // 2ML
  Shift_out_Primary_4(0x00);                          // 4MH
  Shift_out_Primary_1(0x00);                          // 1ML
  Shift_out_Primary_1(0x00);                          // 1LL
  Shift_out_Primary_3(Digit3HL((Temp/0x0A)%0x0A));    // 3HL
  Shift_out_Primary_4(0xE0);                          // 4HH
  Shift_out_Primary_1(0x00);                          // 1LH
  Shift_latch_Primary();
}

void DHT_Show_Temp_Error()                            // -- *C
{
  Shift_out_Primary_2(0xC0);                          // 2HH
  Shift_out_Primary_4(0xD0);                          // 4HL
  Shift_out_Primary_1(0x80);                          // 1HH
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_3(0xE0);                          // 3MH
  Shift_out_Primary_4(0x90);                          // 4ML
  Shift_out_Primary_1(0x00);                          // 1EE
  Shift_out_Primary_2(0x40);                          // 2ML
  Shift_out_Primary_4(0x00);                          // 4MH
  Shift_out_Primary_1(0x00);                          // 1ML
  Shift_out_Primary_1(0x00);                          // 1LL
  Shift_out_Primary_3(0xE0);                          // 3HL
  Shift_out_Primary_4(0xE0);                          // 4HH
  Shift_out_Primary_1(0x00);                          // 1LH
  Shift_latch_Primary();
}

void DHT_Show_Temp()
{
  if (Minus == 1)
  {
    if (Temp > 0x0063)                                // 9.9
      DHT_Show_Temp_Error();
    else
      DHT_Show_Temp_L();
  }
  else
    DHT_Show_Temp_H();
}

// Update Display //////////////////////////////////////////////////////////////

void Display_All()
{
  Show_Second();
  if (((Second > 0x10) && (Second < 0x16)) || ((Second > 0x40) && (Second < 0x46)))
  {
    if (Check == 1)
      DHT_Show_Temp();
    else
      DHT_Show_Error();
  }
  else
  if (((Second > 0x15) && (Second < 0x21)) || ((Second > 0x45) && (Second < 0x51)))
  {
    if (Check == 1)
      DHT_Show_RH();
    else
      DHT_Show_Error();
  }
  else
    Show_Time();
}

// Keys ////////////////////////////////////////////////////////////////////////

unsigned char KeyTemp, Debounce, Set;

void Keys()
{
  if (PORTA.F0 == 1)
  KeyTemp = KeyTemp++;
  delay_ms(5);
  if(KeyTemp > Debounce)
  {
    Hour = Hour++;
    if (Hour % 0x10 > 0x09)
      Hour = Hour + 0x06;
    if(Hour > 0x23)
      Hour = 0x00;
    RTC_Write(0x02, Hour);
    Show_Time();
    KeyTemp = 0;
    Set = 0x00;
  }
  if (PORTA.F2 == 1)
  KeyTemp = KeyTemp++;
  delay_ms(5);
  if(KeyTemp > Debounce)
  {
    Minute = Minute++;
    if (Minute % 0x10 > 0x09)
      Minute = Minute + 0x06;
    if(Minute > 0x59)
      Minute = 0x00;
    RTC_Write(0x01, Minute);
    Show_Time();
    KeyTemp = 0;
    Set = 0x00;
  }
  if (PORTA.F1 == 1)
  KeyTemp = KeyTemp++;
  delay_ms(5);
  if(KeyTemp > Debounce)
  {
    Second = Second++;
    if (Second % 0x10 > 0x09)
      Second = Second + 0x06;
    if(Second > 0x59)
      Second = 0x00;
    RTC_Write(0x00, Second);
    Show_Second();
    KeyTemp = 0;
    Set = 0x00;
  }
}

// Core ////////////////////////////////////////////////////////////////////////

bit SoftINT;

void main(void)
{
  TRISA = 0x07;                 // Key Inputs
  TRISB = 0x00;
  TRISC = 0x71;                 // Unit Inputs
  PORTA = 0x00;
  PORTB = 0x00;
  PORTC = 0x00;
  ADCON0 = 0x00;                // AD Disabled
  ADCON1 = 0x06;                // All inputs are digital
  //T0CON = 0x00;
  //INTCON = 0x00;
  //INTCON2 = 0x00;
  delay_ms(1000);               // Power-Up recovery delay
  KeyTemp = 0;
  Debounce = 0x14;
  Set = 0x64;                   // ???
  SoftINT = 0;
  RTC_init();

  while(1)
  {
    if (PORTC.F0 == 0 && SoftINT == 0)
    {
      Second = RTC_Read(0x00);
      Minute = RTC_Read(0x01);
      Hour = RTC_Read(0x02);
      if (((Minute%0x10 == 0x03) || (Minute%0x10 == 0x08)) && (Second == 0x00))
      {
        DHT_Read_Data();
        //PORTB.F0 = 1;           // Debug
      }
      Display_All();
      SoftINT = 1;
    }
    if (PORTC.F0 == 1 && SoftINT != 0)
    {
      if (Set >= 0x64)
      {
        SoftINT = 0;
        KeyTemp = 0;
      }
      else
        Set ++;
    }
    Keys();
  }
}